package com.gree.mall.manager.controller.workerorder;

import cn.hutool.core.lang.TypeReference;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.manager.annotation.ZfireList;
import com.gree.mall.manager.bean.workorder.PgAppraiseCategoryVO;
import com.gree.mall.manager.constant.Constant;
import com.gree.mall.manager.exception.RemoteServiceException;
import com.gree.mall.manager.helper.ResponseHelper;
import com.gree.mall.manager.logic.workorder.AppraiseRelaLogic;
import com.gree.mall.manager.plus.entity.PgAppraiseCategory;
import com.gree.mall.manager.plus.entity.PgAppraiseConfig;
import com.gree.mall.manager.zfire.bean.ZfireParamBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;

@Slf4j
@RestController
@Api(value = "评价申诉API", tags = {"评价申诉API"})
@RequestMapping(value = "/appraise/apply", produces = "application/json; charset=utf-8")
public class AppraiseApplyController {

    @Resource
    AppraiseRelaLogic appraiseRelaLogic;
    @Resource
    RedisLockRegistry redisLockRegistry;

    @PostMapping("/config/list")
    @ApiOperation(value = "评价配置-列表")
    public ResponseHelper<List<PgAppraiseConfig>> configList() {
        List<PgAppraiseConfig> config = appraiseRelaLogic.configList();
        return ResponseHelper.success(config);
    }

    @ZfireList
    @PostMapping("/category/list")
    @ApiOperation(value = "评价问题类型-列表")
    public ResponseHelper<Page<PgAppraiseCategoryVO>> categoryList(
            @RequestBody ZfireParamBean zfireParamBean
    ) {
        IPage<PgAppraiseCategoryVO> vo = appraiseRelaLogic.categoryList(zfireParamBean);
        return ResponseHelper.success(vo, new TypeReference<PgAppraiseCategoryVO>() {});
    }

    @PostMapping("/category/save")
    @ApiOperation(value = "评价问题类型-保存")
    public ResponseHelper categorySave(PgAppraiseCategory category) {
        appraiseRelaLogic.categorySave(category);
        return ResponseHelper.success();
    }

    @PostMapping("/category/import")
    @ApiOperation(value = "评价问题类型-导入(下载模板:评价问题类别模板.xlsx)")
    public ResponseHelper<String> categoryImportData(
            @RequestPart("file") MultipartFile file
    ) throws Exception {
        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_ORDER);
        if (!obtain.tryLock(10, TimeUnit.SECONDS)) {
            throw new RemoteServiceException("正在导入中，请稍后再试");
        }
        try {
            appraiseRelaLogic.categoryImportData(file);
        } finally {
            obtain.unlock();
        }

        return ResponseHelper.success();
    }
}
